#!/bin/bash -ex
#SBATCH -J analyzeBench
#SBATCH -a 1-7%7
#SBATCH -p hh
#SBATCH -o out.%A_%a
#SBATCH -n 16
#SBATCH -N 1
#SBATCH -t 14-00:00:00

METAEUK_DIR="mmseqs_ebb1_metaeuk_4714"
METAEUK_PATH="${METAEUK_DIR}/build/src/metaeuk"
SCRIPTS_DIR="scripts/"
UNIREF90="YOUR_DIR/uniref90_short_headers.fasta"
FATHER_OUTDIR="annotatedSpecs/mmseqs_ebb1_metaeuk_4714"
## change here to "YES" to chop original scaffolds to typical contig lengths
CHOPPED_MODE="NO"
CONTIG_LENGTHS="YOUR_DIR/TO_contig_lengths.index"

# edit the file bench_specs_paths_new.txt to point at you directory.
# download the files indicated in the file from the NCBI site to your annotatedSpecs directory
LINE_FROM_SED="$(sed "${SLURM_ARRAY_TASK_ID}q;d" YOUR_DIR/annotatedSpecs/bench_specs_paths_new.txt)"
SPECNAME="$(echo $LINE_FROM_SED | awk '{ print $1; }')"
GENOMICS_FILE="$(echo $LINE_FROM_SED | awk '{ print $2; }')"
PROTEIN_TABLE_FILE="$(echo $LINE_FROM_SED | awk '{ print $3; }')"
GFF_FILE="$(echo $LINE_FROM_SED | awk '{ print $4; }')"
PROTEOMICS_FILE="$(echo $LINE_FROM_SED | awk '{ print $5; }')"
SPEC_INPUT_DIR="$(echo $LINE_FROM_SED | awk '{ print $6; }')"


# if in chop-mode
if [ "${CHOPPED_MODE}" == "YES" ]
then
	FATHER_OUTDIR="${FATHER_OUTDIR}/chop/"
	# chop scaffolds to typical lengths
	if [ ! -f "${SPEC_INPUT_DIR}/${SPECNAME}_chopped_scaffolds.fna" ]
	then
		perl "${SCRIPTS_DIR}"/chop_scaffolds.pl "${GENOMICS_FILE}" "${CONTIG_LENGTHS}" "${SPEC_INPUT_DIR}/${SPECNAME}_chopped_scaffolds.fna"
	fi
	GENOMICS_FILE="${SPEC_INPUT_DIR}/${SPECNAME}_chopped_scaffolds.fna"
fi


# prepare files with protein-coding genes with CDS coords and non-coding features
if [ ! -f "${SPEC_INPUT_DIR}/${SPECNAME}_annotated_protein_coords.txt" ]
then
	perl "${SCRIPTS_DIR}"/gff_parser.pl "${GFF_FILE}" "${PROTEOMICS_FILE}" "${SPEC_INPUT_DIR}/${SPECNAME}_annotated_protein_coords.txt" "${SPEC_INPUT_DIR}/${SPECNAME}_annotated_noncoding_coords.txt"
fi

# create output dir and let's go
mkdir -p "${FATHER_OUTDIR}"
mkdir -p "${FATHER_OUTDIR}/${SPECNAME}"

# run MetaEuk on input
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/predEx.dbtype" ]
then
	mkdir -p "${FATHER_OUTDIR}/${SPECNAME}"
	cd "${FATHER_OUTDIR}/${SPECNAME}"
	
	# create DBs from fasta #
	"${METAEUK_PATH}" createdb "${GENOMICS_FILE}" "${SPECNAME}"_contigs --dont-split-seq-by-len --dbtype 2
	"${METAEUK_PATH}" createdb "${UNIREF90}" uniref90 --dbtype 1
	
	# run metaeuk (record mmseqs2 and metaeuk versions, measure time)#
	cd "${METAEUK_DIR}";
	version_mmseqs2=`git submodule status`;
	version_metaeuk=`git log -1 --format="%H"`;
	cd "${FATHER_OUTDIR}/${SPECNAME}"
	echo "version_mmseqs2: $version_mmseqs2, version_metaeuk: $version_metaeuk" > versions_mmseqs2_metaeuk.txt;
	touch _time_start_.txt
	
	# predict:
	"${METAEUK_PATH}" predictexons "${SPECNAME}"_contigs uniref90 predEx tempFolder --metaeuk-eval 0.0001 -e 100 --min-length 20 --metaeuk-tcov 0.6
	# unite un-reduced predictions in a fasta (use target keys instead of accessions):
	"${METAEUK_PATH}" unitesetstofasta "${SPECNAME}"_contigs uniref90 predEx predExTkey.fas --protein 1 --target-key 1
	# reduce redundancy:
	"${METAEUK_PATH}" reduceredundancy predEx predRed predClust
	# unite reduced predictions in a fasta:
	"${METAEUK_PATH}" unitesetstofasta "${SPECNAME}"_contigs uniref90 predRed predRed.fas --protein 1
	# map representatives to un-reduced
	"${METAEUK_PATH}" groupstoacc "${SPECNAME}"_contigs uniref90 predClust predClust.tsv
	
	touch _time_end_.txt
fi

# run MetaEuk on reversed AA fragments (null) #
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/rev_AAs/predEx.dbtype" ]
then
	# Compute Evalues under reversed AA fragments (null) #
	mkdir -p "${FATHER_OUTDIR}/${SPECNAME}"/rev_AAs
	cd "${FATHER_OUTDIR}/${SPECNAME}"/rev_AAs
	echo "version_mmseqs2: $version_mmseqs2, version_metaeuk: $version_metaeuk" > versions_mmseqs2_metaeuk.txt;
	touch _time_start_.txt
	
	# predict:
	"${METAEUK_PATH}" predictexons "${FATHER_OUTDIR}/${SPECNAME}/${SPECNAME}_contigs" "${FATHER_OUTDIR}/${SPECNAME}/uniref90" predEx tempFolder --metaeuk-eval 0.0001 -e 100 --min-length 20 --reverse-fragments 1 --metaeuk-tcov 0.6
	# unite un-reduced predictions in a fasta (use target keys instead of accessions):
	"${METAEUK_PATH}" unitesetstofasta "${FATHER_OUTDIR}/${SPECNAME}/${SPECNAME}_contigs" "${FATHER_OUTDIR}/${SPECNAME}/uniref90" predEx predExTkey.fas --protein 1 --target-key 1
	# reduce redundancy:
	"${METAEUK_PATH}" reduceredundancy predEx predRed predClust
	# unite reduced predictions in a fasta:
	"${METAEUK_PATH}" unitesetstofasta "${FATHER_OUTDIR}/${SPECNAME}/${SPECNAME}_contigs" "${FATHER_OUTDIR}/${SPECNAME}/uniref90" predRed predRed.fas --protein 1
	# map representatives to un-reduced
	"${METAEUK_PATH}" groupstoacc "${FATHER_OUTDIR}/${SPECNAME}/${SPECNAME}_contigs" "${FATHER_OUTDIR}/${SPECNAME}/uniref90" predClust predClust.tsv
	
	touch _time_end_.txt
fi


# prepare directory for Smith-Waterman alignments
mkdir -p "${FATHER_OUTDIR}/${SPECNAME}/SW"

# align MetaEuk group representatives to annotated proteins
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/SW/sw_predRed_to_annotated_prots.m8" ]
then
	cd "${FATHER_OUTDIR}/${SPECNAME}/SW"
	
	# convert proteins fasta to db
	"${METAEUK_PATH}" createdb "${PROTEOMICS_FILE}" annotated_proteins --dont-shuffle --dbtype 1
	
	# convert MetaEuk proteins to db, retaining the order
	"${METAEUK_PATH}" createdb "${FATHER_OUTDIR}/${SPECNAME}/predRed.fas" predRedSeqDB --dont-shuffle --dbtype 1
	
	# search group representatives against annotated proteins ---> "-a" option adds the backtrace and is important for mismatch count!
	"${METAEUK_PATH}" search predRedSeqDB annotated_proteins sw_predRed_to_annotated_prots tmpSearch -e 0.0001 -a --alignment-mode 3 --comp-bias-corr 0 --mask 0
	
	# convert results to m8 format
	"${METAEUK_PATH}" convertalis predRedSeqDB annotated_proteins sw_predRed_to_annotated_prots sw_predRed_to_annotated_prots.m8 --format-output 'query,target,evalue,pident,qcov,tcov,alnlen,mismatch,gapopen,bits'
fi


# align each MetaEuk predicted protein (AAp) to its UniRef90 target (T)
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/SW/sw_predEx_to_target.m8" ]
then
	cd "${FATHER_OUTDIR}/${SPECNAME}/SW"
	
	# convert MetaEuk proteins to db, retaining the order
	"${METAEUK_PATH}" createdb "${FATHER_OUTDIR}/${SPECNAME}/predExTkey.fas" predExTkeySeqDB --dont-shuffle --dbtype 1
	
	# awk command to create prefilter-style DB:
	# predEx_to_target follows the same order as predExTkeySeqDB and each line has a targetKey. 
	# predEx_to_target.index has three columns
	if [ -f predEx_to_target.dbtype ]
	then
		"${METAEUK_PATH}" rmdb predEx_to_target
	fi
	awk -F'[>|]' '/^>/ { data = sprintf("%d\n\0", $2); l = length(data); printf("%s", data) >> "predEx_to_target";  printf("%d\t%d\t%d\n", i, offset, l) >> "predEx_to_target.index"; offset += l; i += 1; }' "${FATHER_OUTDIR}/${SPECNAME}/predExTkey.fas"
	
	# because this is a manipulation to create the prefilter, we need to creat the dbtype
	awk 'BEGIN { printf("%c%c%c%c",7,0,0,0); exit; }' > predEx_to_target.dbtype
	
	# Smith-Waterman of each AAp vs. T (will allow ignoring predictions based on T that is highly similar to AAp):
	"${METAEUK_PATH}" align predExTkeySeqDB "${FATHER_OUTDIR}/${SPECNAME}"/uniref90 predEx_to_target sw_predEx_to_target -e 10000 -a --alignment-mode 3 --comp-bias-corr 0
	
	# convert results to m8 format
	"${METAEUK_PATH}" convertalis predExTkeySeqDB "${FATHER_OUTDIR}/${SPECNAME}"/uniref90 sw_predEx_to_target sw_predEx_to_target.m8 --format-output 'query,target,evalue,pident,qcov,tcov,alnlen,mismatch,gapopen,bits'
fi

# map MetaEuk predictions to annotated genomic features and non-coding features
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_matched_to_annot_prot.tsv" ]
then
	cd "${FATHER_OUTDIR}/${SPECNAME}"
	mkdir -p "${FATHER_OUTDIR}/${SPECNAME}/evaluation"
	
	perl "${SCRIPTS_DIR}/metaeuk_predictions_to_group_and_target.pl" "${FATHER_OUTDIR}/${SPECNAME}/SW/sw_predEx_to_target.m8" "${FATHER_OUTDIR}/${SPECNAME}/predClust.tsv" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predToMinSeqid.tsv"
	
	perl "${SCRIPTS_DIR}/evaluate_metaeuk_group_representatives.pl" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predToMinSeqid.tsv" "${FATHER_OUTDIR}/${SPECNAME}/SW/sw_predRed_to_annotated_prots.m8" "${FATHER_OUTDIR}/${SPECNAME}/predRed.fas" "${SPEC_INPUT_DIR}/${SPECNAME}_annotated_protein_coords.txt" "${SPEC_INPUT_DIR}/${SPECNAME}_annotated_noncoding_coords.txt" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_matched_to_annot_prot.tsv" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_map_to_annots.tsv"
fi

# extract the SW info from alignment of MetaEuk proteins to their targets
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_SW.tsv" ]
then
	cd "${FATHER_OUTDIR}/${SPECNAME}"
	mkdir -p "${FATHER_OUTDIR}/${SPECNAME}/evaluation"
	
	perl "${SCRIPTS_DIR}/get_sw_to_target.pl" "${FATHER_OUTDIR}/${SPECNAME}/SW/sw_predEx_to_target.m8" "${FATHER_OUTDIR}/${SPECNAME}/predRed.fas" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predEx_SW.tsv" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_SW.tsv"
fi

# add the mapping status to the SW alignment of MetaEuk proteins to their targets
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_SW_and_map_to_annots.tsv" ]
then
	cd "${FATHER_OUTDIR}/${SPECNAME}"
	mkdir -p "${FATHER_OUTDIR}/${SPECNAME}/evaluation"
	
	perl "${SCRIPTS_DIR}/add_mapping_to_sw.pl" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_map_to_annots.tsv" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_SW.tsv" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_SW_and_map_to_annots.tsv"
fi

# compute exon length distributions 
if [ ! -f "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_exon_num_len.tsv" ]
then
	cd "${FATHER_OUTDIR}/${SPECNAME}"
	mkdir -p "${FATHER_OUTDIR}/${SPECNAME}/evaluation"
	
	perl "${SCRIPTS_DIR}/get_exon_num_and_length_dist.pl" "${FATHER_OUTDIR}/${SPECNAME}/predRed.fas" "${FATHER_OUTDIR}/${SPECNAME}/evaluation/predRed_exon_num_len.tsv"
fi
