library(ggplot2)
library(dplyr)
library(scales)
library(cowplot)

theme_set(theme_classic(
  base_size = 9
))

source_names = list(
  "MERC" = "MERC" ,
  "MMETSP" = "MMETSP",
  "UNICLUST50" = "Uniclust50"
)

variable_labeller = function(variable,value){
  return(source_names[value])
}

args <- commandArgs(TRUE)
lens_file <- as.character(args[1])
path_for_plot <- as.character(args[2])

## for debug:
lens_file="tara/run_011/predRed_vs_EUK_REFS_ownT_some_cols.tsv"
path_for_plot="tara/run_011/plots/predRed_Tara5kbEukRep_target_seqid_cov"

dir.create(file.path("tara/run_011/plots/"), showWarnings = FALSE)

path_for_plot_png = paste(path_for_plot,".png",sep = "")
path_for_plot_pdf = paste(path_for_plot,".pdf",sep = "")


### read data ###
lens_df = read.table(file = lens_file, header = T, sep = "\t")
summary(lens_df$LEN_RATIO)
summary(lens_df$TCOV)
summary(lens_df$SEQID)

lens_df$NUM_EXONS_CAT = 0
lens_df$NUM_EXONS_CAT[which(lens_df$NUM_EXONS > 1)] = 1

# for tests - make smaller...
#lens_df = lens_df[1:10000,]


### draw target coverage ###
fig_t_cov = ggplot(lens_df, aes(x = as.factor(NUM_EXONS_CAT), y = TCOV)) + 
  facet_wrap(~ SOURCE, labeller = labeller(SOURCE = variable_labeller)) +
  geom_violin(aes(color = as.factor(NUM_EXONS_CAT)), position = position_dodge(width = 1), lwd=1) + 
  scale_color_manual(values = c("#83DDDF", "#F9BDB8"), breaks = c(0,1), labels = c("1", "2+")) +
  geom_boxplot(aes(fill = as.factor(NUM_EXONS_CAT)), alpha=0.6, width=0.3, outlier.shape = NA, position = position_dodge(width = 1)) +
  scale_fill_manual(values = c("#83DDDF", "#F9BDB8"), breaks = c(0,1), labels = c("1", "2+")) +
  scale_x_discrete(breaks = c(0,1), labels = c("1", "2+")) +
  scale_y_continuous(expand=c(0,0), limits = c(0,1), breaks = seq(0,1,by=0.25), labels = scales::percent(seq(0,1,by=0.25))) +
  xlab("# putative exons") +
  ylab("Target coverage") +
  theme(legend.position = "none") +
  theme(strip.background = element_blank())
  
### seqid ###

# manipulate data #
lens_df$SEQID = round(lens_df$SEQID, digits = 2)

lens_MERC_df = lens_df[which(lens_df$SOURCE == "MERC"),]
lens_MERC_df = lens_MERC_df[rev(order(lens_MERC_df$SEQID)),]
lens_MERC_df$N_PREDS_GE_SEQID = rev(order(lens_MERC_df$SEQID))

lens_MMETSP_df = lens_df[which(lens_df$SOURCE == "MMETSP"),]
lens_MMETSP_df = lens_MMETSP_df[rev(order(lens_MMETSP_df$SEQID)),]
lens_MMETSP_df$N_PREDS_GE_SEQID = rev(order(lens_MMETSP_df$SEQID))

lens_UNICLUST50_df = lens_df[which(lens_df$SOURCE == "UNICLUST50"),]
lens_UNICLUST50_df = lens_UNICLUST50_df[rev(order(lens_UNICLUST50_df$SEQID)),]
lens_UNICLUST50_df$N_PREDS_GE_SEQID = rev(order(lens_UNICLUST50_df$SEQID))

joined_df = rbind(lens_MERC_df,lens_MMETSP_df,lens_UNICLUST50_df)
tot_num_preds = dim(joined_df)[1]

joined_df2 = joined_df %>%
  group_by(SEQID, SOURCE) %>%
  summarise(percentage = max(N_PREDS_GE_SEQID)/tot_num_preds)

fig_seqid = ggplot(joined_df2, aes(x=SEQID, y=percentage, fill=SOURCE)) +
  geom_area(stat="identity") +
  scale_fill_manual(values = c("#F06165", "#70A2C9", "#7AC98A"), breaks = c("MERC", "MMETSP", "UNICLUST50"), labels = c("MERC", "MMETSP", "Uniclust50")) +
  scale_x_continuous(expand = c(0,0), limits = c(0.25,1.01), breaks = seq(0,1,by=0.25), labels = scales::percent(seq(0,1,by=0.25))) +
  scale_y_continuous(expand = c(0,0), limits = c(0,1.05), breaks = seq(0,1,by=0.25), labels = scales::percent(seq(0,1,by=0.25))) +
  theme(
    legend.position = c(0.64,0.94),
    legend.direction = "horizontal",
    legend.title = element_blank(),
    legend.key.width = unit(2, "mm"),
    legend.key.height = unit(1, "mm"),
    legend.spacing.x = unit(0.5, "mm")
  ) +
  xlab("Seq. id. to target") +
  ylab("Predictions")


#### arrange #####

seqid_tcov_grid = plot_grid(
  fig_seqid,
  fig_t_cov,
  labels = c("A", "B"),
  nrow=2, ncol=1, rel_heights = c(0.5,0.5)
)

ggsave(path_for_plot_png, plot = seqid_tcov_grid, width = 85, height = 90, units = c("mm"))
ggsave(path_for_plot_pdf, plot = seqid_tcov_grid, width = 85, height = 90, units = c("mm"))


